@@ -79,9 +79,11 @@ import sys
 import types
 from collections import OrderedDict, defaultdict
 from itertools import permutations, zip_longest
+from keyword import iskeyword
 from string import ascii_lowercase
 from textwrap import dedent, indent
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     Dict,
@@ -100,8 +102,10 @@ import black
 from hypothesis import Verbosity, find, settings, strategies as st
 from hypothesis.errors import InvalidArgument
 from hypothesis.internal.compat import get_type_hints
-from hypothesis.internal.reflection import is_mock
+from hypothesis.internal.reflection import get_signature, is_mock
 from hypothesis.internal.validation import check_type
+from hypothesis.provisional import domains
+from hypothesis.strategies._internal.collections import ListStrategy
 from hypothesis.strategies._internal.core import BuildsStrategy
 from hypothesis.strategies._internal.flatmapped import FlatMapStrategy
 from hypothesis.strategies._internal.lazy import LazyStrategy, unwrap_strategies
@@ -116,7 +120,8 @@ from hypothesis.utils.conventions import infer
 
 if sys.version_info >= (3, 10):  # pragma: no cover
     from types import EllipsisType as InferType
-
+elif TYPE_CHECKING:
+    from builtins import ellipsis as InferType
 else:
     InferType = type(Ellipsis)
 
@@ -219,27 +224,6 @@ def _exceptions_from_docstring(doc: str) -> Tuple[Type[Exception], ...]:
     return tuple(_dedupe_exceptions(tuple(raises)))
 
 
-# Simple strategies to guess for common argument names - we wouldn't do this in
-# builds() where strict correctness is required, but we only use these guesses
-# when the alternative is nothing() to force user edits anyway.
-#
-# This table was constructed manually after skimming through the documentation
-# for the builtins and a few stdlib modules.  Future enhancements could be based
-# on analysis of type-annotated code to detect arguments which almost always
-# take values of a particular type.
-_GUESS_STRATEGIES_BY_NAME = (
-    (st.integers(0, 32), ["ndims", "ndigits"]),
-    (st.booleans(), ["keepdims"]),
-    (st.text(), ["name", "filename", "fname", "text"]),
-    (st.floats(), ["real", "imag"]),
-    (st.functions(), ["function", "func", "f"]),
-    (st.functions(returns=st.booleans(), pure=True), ["pred", "predicate"]),
-    (st.iterables(st.integers()) | st.iterables(st.text()), ["iterable"]),
-    (st.builds(object), ["object"]),
-    (st.integers() | st.floats() | st.fractions(), ["number"]),
-)
-
-
 def _type_from_doc_fragment(token: str) -> Optional[type]:
     # Special cases for "integer" and for numpy array-like and dtype
     if token == "integer":
@@ -333,14 +317,124 @@ def _strategy_for(
         # failures in cases like the `flags` argument to regex functions.
         # Better in to keep it simple, and let the user elaborate if desired.
         return st.just(param.default)
-    # If there's no annotation and no default value, we check against a table
-    # of guesses of simple strategies for common argument names.
-    if "string" in param.name and "as" not in param.name:
+    return _guess_strategy_by_argname(name=param.name.lower())
+
+
+# fmt: off
+BOOL_NAMES = (
+    "keepdims", "verbose", "debug", "force", "train", "training", "trainable", "bias",
+    "shuffle", "show", "load", "pretrained", "save", "overwrite", "normalize",
+    "reverse", "success", "enabled", "strict", "copy", "quiet", "required", "inplace",
+    "recursive", "enable", "active", "create", "validate", "refresh", "use_bias",
+)
+POSITIVE_INTEGER_NAMES = (
+    "width", "size", "length", "limit", "idx", "stride", "epoch", "epochs", "depth",
+    "pid", "steps", "iteration", "iterations", "vocab_size", "ttl", "count",
+)
+FLOAT_NAMES = (
+    "real", "imag", "alpha", "theta", "beta", "sigma", "gamma", "angle", "reward",
+    "tau", "temperature",
+)
+STRING_NAMES = (
+    "text", "txt", "password", "label", "prefix", "suffix", "desc", "description",
+    "str", "pattern", "subject", "reason", "comment", "prompt", "sentence", "sep",
+)
+# fmt: on
+
+
+def _guess_strategy_by_argname(name: str) -> st.SearchStrategy:
+    """
+    If all else fails, we try guessing a strategy based on common argument names.
+
+    We wouldn't do this in builds() where strict correctness is required, but for
+    the ghostwriter we accept "good guesses" since the user would otherwise have
+    to change the strategy anyway - from `nothing()` - if we refused to guess.
+
+    A "good guess" is _usually correct_, and _a reasonable mistake_ if not.
+    The logic below is therefore based on a manual reading of the builtins and
+    some standard-library docs, plus the analysis of about three hundred million
+    arguments in https://github.com/HypothesisWorks/hypothesis/issues/3311
+    """
+    # Special-cased names
+    if name in ("function", "func", "f"):
+        return st.functions()
+    if name in ("pred", "predicate"):
+        return st.functions(returns=st.booleans(), pure=True)
+    if name in ("iterable",):
+        return st.iterables(st.integers()) | st.iterables(st.text())
+    if name in ("list", "lst", "ls"):
+        return st.lists(st.nothing())
+    if name in ("object",):
+        return st.builds(object)
+    if "uuid" in name:
+        return st.uuids().map(str)
+
+    # Names which imply the value is a boolean
+    if name.startswith("is_") or name in BOOL_NAMES:
+        return st.booleans()
+
+    # Names which imply that the value is a number, perhaps in a particular range
+    if name in ("amount", "threshold", "number", "num"):
+        return st.integers() | st.floats()
+
+    if name in ("port",):
+        return st.integers(0, 2**16 - 1)
+    if (
+        name.endswith("_size")
+        or (name.endswith("size") and "_" not in name)
+        or re.fullmatch(r"n(um)?_[a-z_]*s", name)
+        or name in POSITIVE_INTEGER_NAMES
+    ):
+        return st.integers(min_value=0)
+    if name in ("offset", "seed", "dim", "total", "priority"):
+        return st.integers()
+
+    if name in ("learning_rate", "dropout", "dropout_rate", "epsilon", "eps", "prob"):
+        return st.floats(0, 1)
+    if name in ("lat", "latitude"):
+        return st.floats(-90, 90)
+    if name in ("lon", "longitude"):
+        return st.floats(-180, 180)
+    if name in ("radius", "tol", "tolerance", "rate"):
+        return st.floats(min_value=0)
+    if name in FLOAT_NAMES:
+        return st.floats()
+
+    # Names which imply that the value is a string
+    if name in ("host", "hostname"):
+        return domains()
+    if name in ("email",):
+        return st.emails()
+    if name in ("word", "slug", "api_key"):
+        return st.from_regex(r"\w+", fullmatch=True)
+    if name in ("char", "character"):
+        return st.characters()
+
+    if (
+        "file" in name
+        or "path" in name
+        or name.endswith("_dir")
+        or name in ("fname", "dir", "dirname", "directory", "folder")
+    ):
+        # Common names for filesystem paths: these are usually strings, but we
+        # don't want to make strings more convenient than pathlib.Path.
+        return st.nothing()
+
+    if (
+        name.endswith("_name")
+        or (name.endswith("name") and "_" not in name)
+        or ("string" in name and "as" not in name)
+        or name.endswith("label")
+        or name in STRING_NAMES
+    ):
         return st.text()
-    for strategy, names in _GUESS_STRATEGIES_BY_NAME:
-        if param.name in names:
-            assert isinstance(strategy, st.SearchStrategy)
-            return strategy
+
+    # Last clever idea: maybe we're looking a plural, and know the singular:
+    if re.fullmatch(r"\w*[^s]s", name):
+        elems = _guess_strategy_by_argname(name[:-1])
+        if not elems.is_empty:
+            return st.lists(elems)
+
     # And if all that failed, we'll return nothing() - the user will have to
     # fill this in by hand, and we'll leave a comment to that effect later.
     return st.nothing()
@@ -350,7 +444,7 @@ def _get_params(func: Callable) -> Dict[str, inspect.Parameter]:
     """Get non-vararg parameters of `func` as an ordered dict."""
     var_param_kinds = (inspect.Parameter.VAR_POSITIONAL, inspect.Parameter.VAR_KEYWORD)
     try:
-        params = list(inspect.signature(func).parameters.values())
+        params = list(get_signature(func).parameters.values())
     except Exception:
         if (
             isinstance(func, (types.BuiltinFunctionType, types.BuiltinMethodType))
@@ -369,14 +463,16 @@ def _get_params(func: Callable) -> Dict[str, inspect.Parameter]:
             kind: inspect._ParameterKind = inspect.Parameter.POSITIONAL_ONLY
             for arg in args.split(", "):
                 arg, *_ = arg.partition("=")
+                arg = arg.strip()
                 if arg == "/":
                     kind = inspect.Parameter.POSITIONAL_OR_KEYWORD
                     continue
-                if arg.startswith("*"):
+                if arg.startswith("*") or arg == "...":
                     kind = inspect.Parameter.KEYWORD_ONLY
                     continue  # we omit *varargs, if there are any
-                if arg.startswith("**"):
-                    break  # and likewise omit **varkw
+                if iskeyword(arg.lstrip("*")) or not arg.lstrip("*").isidentifier():
+                    print(repr(args))
+                    break  # skip all subsequent params if this name is invalid
                 params.append(inspect.Parameter(name=arg, kind=kind))
 
         elif _is_probably_ufunc(func):
@@ -496,6 +592,8 @@ def _imports_for_strategy(strategy):
             return _imports_for_object(strategy._LazyStrategy__args[0])
         elif _get_module(strategy.function).startswith("hypothesis.extra."):
             return {(_get_module(strategy.function), strategy.function.__name__)}
+            module = _get_module(strategy.function).replace("._array_helpers", ".numpy")
+            return {(module, strategy.function.__name__)}
 
     imports = set()
     strategy = unwrap_strategies(strategy)
@@ -529,6 +627,9 @@ def _imports_for_strategy(strategy):
         for obj in strategy.elements:
             imports |= _imports_for_object(obj)
 
+    if isinstance(strategy, ListStrategy):
+        imports |= _imports_for_strategy(strategy.element_strategy)
+
     return imports
 
 
@@ -648,11 +749,12 @@ def _make_test_body(
     assertions: str = "",
     style: str,
     given_strategies: Optional[Mapping[str, Union[str, st.SearchStrategy]]] = None,
+    imports: Optional[ImportSet] = None,
 ) -> Tuple[ImportSet, str]:
     # A set of modules to import - we might add to this later.  The import code
     # is written later, so we can have one import section for multiple magic()
     # test functions.
-    imports = {_get_module(f) for f in funcs}
+    imports = (imports or set()) | {_get_module(f) for f in funcs}
 
     # Get strategies for all the arguments to each function we're testing.
     with _with_any_registered():
@@ -791,10 +893,15 @@ def magic(
     for thing in modules_or_functions:
         if callable(thing):
             functions.add(thing)
+            # class need to be added for exploration
+            if inspect.isclass(thing):
+                funcs: List[Optional[Any]] = [thing]
+            else:
+                funcs = []
         elif isinstance(thing, types.ModuleType):
             if hasattr(thing, "__all__"):
                 funcs = [getattr(thing, name, None) for name in thing.__all__]
-            else:
+            elif hasattr(thing, "__package__"):
                 pkg = thing.__package__
                 funcs = [
                     v
@@ -806,22 +913,35 @@ def magic(
                 ]
                 if pkg and any(getattr(f, "__module__", pkg) == pkg for f in funcs):
                     funcs = [f for f in funcs if getattr(f, "__module__", pkg) == pkg]
-            for f in funcs:
-                try:
-                    if (
-                        (not is_mock(f))
-                        and callable(f)
-                        and _get_params(f)
-                        and not isinstance(f, enum.EnumMeta)
-                    ):
-                        functions.add(f)
-                        if getattr(thing, "__name__", None):
-                            KNOWN_FUNCTION_LOCATIONS[f] = thing.__name__
-                except (TypeError, ValueError):
-                    pass
         else:
             raise InvalidArgument(f"Can't test non-module non-callable {thing!r}")
 
+        for f in list(funcs):
+            if inspect.isclass(f):
+                funcs += [
+                    v.__get__(f)
+                    for k, v in vars(f).items()
+                    if hasattr(v, "__func__")
+                    and not is_mock(v)
+                    and not k.startswith("_")
+                ]
+        for f in funcs:
+            try:
+                if (
+                    (not is_mock(f))
+                    and callable(f)
+                    and _get_params(f)
+                    and not isinstance(f, enum.EnumMeta)
+                ):
+                    functions.add(f)
+                    if getattr(thing, "__name__", None):
+                        if inspect.isclass(thing):
+                            KNOWN_FUNCTION_LOCATIONS[f] = thing.__module__
+                        else:
+                            KNOWN_FUNCTION_LOCATIONS[f] = thing.__name__
+            except (TypeError, ValueError):
+                pass
+
     imports = set()
     parts = []
 
@@ -1403,13 +1523,17 @@ def _make_ufunc_body(func, *, except_, style):
         shapes = npst.mutually_broadcastable_shapes(num_shapes=func.nin)
     else:
         shapes = npst.mutually_broadcastable_shapes(signature=func.signature)
+    shapes.function.__module__ = npst.__name__
 
     body = """
     input_shapes, expected_shape = shapes
     input_dtypes, expected_dtype = types.split("->")
-    array_st = [npst.arrays(d, s) for d, s in zip(input_dtypes, input_shapes)]
+    array_strats = [
+        arrays(dtype=dtp, shape=shp, elements={{"allow_nan": True}})
+        for dtp, shp in zip(input_dtypes, input_shapes)
+    ]
 
-    {array_names} = data.draw(st.tuples(*array_st))
+    {array_names} = data.draw(st.tuples(*array_strats))
     result = {call}
     """.format(
         array_names=", ".join(ascii_lowercase[: func.nin]),
@@ -1435,4 +1559,5 @@ def _make_ufunc_body(func, *, except_, style):
         ghost="ufunc" if func.signature is None else "gufunc",
         style=style,
         given_strategies={"data": st.data(), "shapes": shapes, "types": types},
+        imports={("hypothesis.extra.numpy", "arrays")},
     )
